--[[
	GuildBankSpoofer attempts to fool addons into thinking your guild bank is your personal bank.
	I use Reagent Restocker's feature to overstock items to bank. I'd like it to move overstocked items to guild bank and withdraw them when they're needed in my bag.
]]

local folder, core = ...
_GBS = core --needs to be made global

--[===[@debug@
--@end-debug@]===]
local LibStub = LibStub
local _G = _G
local chatFrame
local pairs = pairs
local tostring = tostring
local table_getn = table.getn
local InterfaceOptionsFrame_OpenToCategory = InterfaceOptionsFrame_OpenToCategory
local _
local GetItemInfo = GetItemInfo
local GetNumGuildBankTabs = GetNumGuildBankTabs
local ipairs = ipairs
local GetFramesRegisteredForEvent = GetFramesRegisteredForEvent
local table_concat = table.concat
local GetCursorInfo = GetCursorInfo
local CreateFrame = CreateFrame
local strsplit = strsplit
local tonumber = tonumber

core.title		= "Guild Bank Spoofer"
core.version	= GetAddOnMetadata(folder, "X-Curse-Packaged-Version") or ""
core.titleFull	= core.title.." "..core.version
core.addonDir = "Interface\\AddOns\\"..folder.."\\"

LibStub("AceAddon-3.0"):NewAddon(core, folder, "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0", "AceTimer-3.0") 

core.L = LibStub("AceLocale-3.0"):GetLocale(folder, true)
local L = core.L

local P --db.profile
core.defaultSettings = {
	profile = {
	},
}

local spoofing = false --User has clicked spoofing button. Our events won't fake bank events unless this is enabled.

local slotsPerBankTab = 98 --slots per tab.


local regEvents = {
--~ 	"ACTIONBAR_SLOT_CHANGED",
	"GUILDBANKFRAME_OPENED",
	"GUILDBANKFRAME_CLOSED",
--~ 	"BAG_UPDATE",
	"PLAYERBANKBAGSLOTS_CHANGED",
	"PLAYERBANKSLOTS_CHANGED",
	"GUILDBANKBAGSLOTS_CHANGED",
	"ITEM_LOCK_CHANGED",
	"GUILDBANK_ITEM_LOCK_CHANGED",
}
local coreOpts

function core:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("GBS_DB", core.defaultSettings)
--~ 	self.CoreOptionsTable.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileDeleted", "OnProfileChanged")
	self:RegisterChatCommand("gbs", "MySlashProcessorFunc")
	
	chatFrame = _G["ChatFrame1"]
	
	--[[
	local config = LibStub("AceConfig-3.0")
	local dialog = LibStub("AceConfigDialog-3.0")
	config:RegisterOptionsTable(folder, self.CoreOptionsTable)
	coreOpts = dialog:AddToBlizOptions(folder, self.titleFull)
	]]
	
--~ 	hooksecurefunc("PickupAction", core.InsecurelyUpdateAction)
--~ 	hooksecurefunc("PlaceAction", core.InsecurelyUpdateAction)

	self:RawHook("GetContainerNumSlots", true)
	self:RawHook("GetContainerItemLink", true)
	self:RawHook("GetContainerItemInfo", true)
	self:RawHook("SplitContainerItem", true)
	self:RawHook("PickupContainerItem", true)
	self:RawHook("PutItemInBag", true)
	self:RawHook("GetItemCount", true)
	
	
--~ 	self:RawHook("UseContainerItem", true)

end



----------------------------------------------------------------------
function core:OnProfileChanged(...)									--
-- User has reset proflie, so we reset our spell exists options.	--
----------------------------------------------------------------------
	-- Shut down anything left from previous settings
	self:Disable()
	-- Enable again with the new settings
	self:Enable()
end

function core:OnEnable()
	P = self.db.profile

	for i, event in pairs(regEvents) do 
		self:RegisterEvent(event)
	end
end



function core:OnDisable()
--~ 	timerFrame:SetScript("OnUpdate", nil)
end


local DEBUG = false
--[===[@debug@
DEBUG = true
--@end-debug@]===]

local strWhiteBar		= "|cffffff00 || |r" -- a white bar to seperate the debug info.
local colouredName		= "|cff7f7f7f{|r|cffff0000GBS|r|cff7f7f7f}|r "
local function echo(...)
	local tbl  = {...}
	local msg = tostring(tbl[1])
	for i=2,table_getn(tbl) do 
		msg = msg..strWhiteBar..tostring(tbl[i])
	end
	

	local cf = chatFrame
	if cf then
		cf:AddMessage(colouredName..msg,.7,.7,.7)
	end
end
core.echo = echo

--~ local whiteText			= "|cffffffff%s|r"
local strDebugFrom		= "|cffffff00[%s]|r" --Yellow function name. help pinpoint where the debug msg is from.
-----------------------------
local function Debug(from, ...)	--
-- simple print function.	--
------------------------------
	if DEBUG == false then
		return 
	end
	local tbl  = {...}
	local msg = tostring(tbl[1])
	for i=2,table_getn(tbl) do 
		msg = msg..strWhiteBar..tostring(tbl[i])
	end

	
	echo(strDebugFrom:format(from).." "..msg)
end
core.Debug = Debug

----------------------------------------------
function core:MySlashProcessorFunc(input)	--
-- /da function brings up the UI options.	--
----------------------------------------------
	echo("MySlashProcessorFunc")
	InterfaceOptionsFrame_OpenToCategory(coreOpts)
end

--[[
	Real bank containers = 5 - 11
	
	
	numTabs = GetNumGuildBankTabs()

	
]]


--[[
local testFrame = CreateFrame("Frame")
testFrame:SetScript("OnEvent", function(frame, event, ...)
	local container = 5
	Debug("testFrame", event, "slots:"..tostring( _G.GetContainerNumSlots(container) ) )
	
	
	local slots = _G.GetContainerNumSlots(container)
	local itemLink, count
	for slot = 1, slots, 1 do
		itemLink = _G.GetContainerItemLink(container, slot)
        if itemLink then
           _,count = _G.GetContainerItemInfo(container, slot)
			Debug("testFrame", slot, itemLink, count)
		end
	end
end)
testFrame:RegisterEvent("BANKFRAME_OPENED")]]




function core:ItemLinkToId(itemLink) -- Itemlink, returns the itemID
	local _, _, itemString = itemLink:find("^|c%x+|H(.+)|h%[.+%]")
	if itemString then
		local _, itemId = strsplit(":", itemString)
		return tonumber(itemId)
	end
	return nil
end

local lockEventTimer = false

--- Events
function core:SpoofBank()--		/run _GBS:SpoofBank()
	spoofing = true
	for i,frame in ipairs({GetFramesRegisteredForEvent("BANKFRAME_OPENED")}) do
--~ 		frame:GetScript("OnEvent")
		if frame ~= _G.BankFrame then
			frame:GetScript("OnEvent")(frame, "BANKFRAME_OPENED")
		end
	end
end

local ourSpoofingButton = false

function core:GUILDBANKFRAME_OPENED(event, ...)
--~ 	Debug(event, "/run _GBS:SpoofBank()")
	if not ourSpoofingButton then
		local f = CreateFrame("Button", nil, _G.GuildBankFrame, "UIPanelButtonTemplate")
		f:SetWidth(60)
		f:SetHeight(21)
		
		f:SetText("Spoof")
		
		f:SetPoint("BOTTOMLEFT", _G.GuildBankFrame, "BOTTOMLEFT", 25, 37)
		
		f:SetScript("OnClick", function(frame, ...)
--~ 			print("Hi!")
			Debug("Spoof Click","Telling other frames your bank has been opened.")
			
			self:CancelTimer(lockEventTimer, true)
			self:SpoofBank()
		end)
		
		ourSpoofingButton = f 
	end
--~ 	
end
function core:GUILDBANKFRAME_CLOSED(event, ...)
--~ 	Debug(event, ...)
	
	for i,frame in ipairs({GetFramesRegisteredForEvent("BANKFRAME_CLOSED")}) do
--~ 		frame:GetScript("OnEvent")
--~ 		if frame ~= _G.BankFrame then
			frame:GetScript("OnEvent")(frame, "BANKFRAME_CLOSED")
--~ 		end
	end
	spoofing = false
	self:CancelTimer(lockEventTimer, true)
end

--[[function core:BAG_UPDATE(event, ...)
	Debug(event, ...)
end]]

function core:PLAYERBANKBAGSLOTS_CHANGED(event, ...)
--~ 	Debug(event, ...)
end
function core:PLAYERBANKSLOTS_CHANGED(event, ...)
--~ 	Debug(event, ...)
end

function core:GUILDBANKBAGSLOTS_CHANGED(event, ...)

	if spoofing == true then
--~ 		Debug(event, ...)
		for i,frame in ipairs({GetFramesRegisteredForEvent("PLAYERBANKSLOTS_CHANGED")}) do
			frame:GetScript("OnEvent")(frame, "PLAYERBANKSLOTS_CHANGED")
		end
	end
end

function core:StartLockEventTimer()
	self:CancelTimer(lockEventTimer, true)
	lockEventTimer = self:ScheduleRepeatingTimer("FireLockEvent", 1, "ITEM_LOCK_CHANGED")
end

function core:ITEM_LOCK_CHANGED(event, ...)
--~ 	Debug(event, ..., "-", GetCursorInfo(), "-", GetCursorInfo())
	
	if spoofing == true then
		self:StartLockEventTimer()
	end
end



function core:GUILDBANK_ITEM_LOCK_CHANGED(event, ...)
--~ 	Debug(event, ..., "-", GetCursorInfo(), "-", GetCursorInfo())
	if spoofing == true then
--~ 		Debug(event, ...)
--~ 		for i,frame in ipairs({GetFramesRegisteredForEvent("ITEM_LOCK_CHANGED")}) do
--~ 			frame:GetScript("OnEvent")(frame, "ITEM_LOCK_CHANGED")
--~ 		end
		self:StartLockEventTimer()
	end
end

--[[
	Reagent Restocker uses ITEM_LOCK_CHANGED but if I fire it on GUILDBANK_ITEM_LOCK_CHANGED sometimes items are still locked causing RR to stop in it's tracks.
	So, I'm using a timer. =/
]]
function core:FireLockEvent(event)
--~ 	Debug("FireLockEvent", event)
	core:UnregisterEvent(event)
	for i,frame in ipairs({GetFramesRegisteredForEvent(event)}) do
		--No sense firing our own frame or blizzard UI frames. =/
		if	frame ~= core and
			frame ~= _G.BankFrame and
			frame ~= _G.ContainerFrame and
			frame ~= _G.PaperDollItemSlotButton then
		
			frame:GetScript("OnEvent")(frame, event)
		end
	end
	core:RegisterEvent(event)
end
--		/run _GBS:SpoofBank()
--[[
	Bank = -1, 5,6,7,8,9,10,11
]]	

function core:containerToTab(container)
	if container == -1 then
		return 1
	end
	if container >= 5 then
		return container - 3
	end
	return nil
end

--Hooks
--		/run _GBS:SpoofBank()
function core:GetContainerNumSlots(...)
	local container = ...
	if _G.GuildBankFrame and _G.GuildBankFrame:IsShown() and container >= 5 then
		local tab = self:containerToTab(container)
--~ 		Debug("GetContainerNumSlots","Guild bank is open", container, tab, "tabs:"..tostring(GetNumGuildBankTabs()))
		if GetNumGuildBankTabs() >= tab then
			return slotsPerBankTab
		end
	end
	return self.hooks.GetContainerNumSlots(...)
end

function core:GetContainerItemLink(...)
	local container, slot = ...

	if _G.GuildBankFrame and _G.GuildBankFrame:IsShown() and container >= 5 then
		local tab = self:containerToTab(container)
--~ 		if _G.GetGuildBankItemLink(tab, slot) then
--~ 			Debug("GetContainerItemLink", container, slot,"tab:"..tostring(tab), _G.GetGuildBankItemLink(tab, slot))
--~ 		end
		return _G.GetGuildBankItemLink(tab, slot)
	end
	return self.hooks.GetContainerItemLink(...)
end

--~ texture, count, locked, quality,  readable, lootable, link =  GetContainerItemInfo(container, slot)
--~ texture, count, locked =  GetGuildBankItemInfo(tab, slot)
local lastLink = false

function core:GetContainerItemInfo(...)
	local container, slot = ...
	

	if _G.GuildBankFrame and _G.GuildBankFrame:IsShown() and container >= 5 then
		local tab = self:containerToTab(container)
		
		local texture, count, locked =  _G.GetGuildBankItemInfo(tab, slot)
		local itemLink = _G.GetGuildBankItemLink(tab, slot)
--~ 		if itemLink ~= lastLink then
--~ 			lastLink = itemLink
--~ 			Debug("GetContainerItemInfo", ..., itemLink)
--~ 		end
		if itemLink then
			local _, _, quality = GetItemInfo(itemLink)
			local readable = nil
			local lootable = nil
			return texture, count, locked, quality, readable, lootable, itemLink
		end
		return nil
	end

	return self.hooks.GetContainerItemInfo(...)
end


function core:SplitContainerItem(...)
	local container, slot, amount = ...
	
	if _G.GuildBankFrame and _G.GuildBankFrame:IsShown() and container >= 5 then
		local tab = self:containerToTab(container)
--~ 		Debug("SplitContainerItem", "SplitGuildBankItem", tab, slot, amount)
		return _G.SplitGuildBankItem(tab, slot, amount)
	end
	
--~ 	Debug("SplitContainerItem", container, slot)
	return self.hooks.SplitContainerItem(...)
end

function core:PickupContainerItem(...)
	local container, slot = ...
--~ 	
	if _G.GuildBankFrame and _G.GuildBankFrame:IsShown() and container >= 5 then
		local tab = self:containerToTab(container)
--~ 		Debug("PickupContainerItem", "PickupGuildBankItem", tab, slot)
		return _G.PickupGuildBankItem(tab, slot)
	end
	
--~ 	Debug("PickupContainerItem", container, slot)
	return self.hooks.PickupContainerItem(...)
end

function core:PutItemInBag(...)
	local container = ...
	if _G.GuildBankFrame and _G.GuildBankFrame:IsShown() and container >= 5 then
		local tab = self:containerToTab(container)
		
		
	end
	return self.hooks.PutItemInBag(...)
end

function core:GetItemCount(...)
	local itemId, includeBank, includeCharges = ...
	
	--[[
	if includeBank ~= false then
		--Since incoming itemID can be a name/itemLink, I wanna make sure I'm getting a link.
		local _, reqItemLink =  GetItemInfo(itemId)
		if reqItemLink then
			itemId = self:ItemLinkToId(reqItemLink)
	
			if _G.GuildBankFrame and _G.GuildBankFrame:IsShown() then
				local totalCount = self.hooks.GetItemCount(itemId, false, true) or 0
				local itemLink, itemID, count
				for t=1, GetNumGuildBankTabs() do 
				
					for s=1, slotsPerBankTab do 
						itemLink = _G.GetGuildBankItemLink(t, s)
						if itemLink then
							itemID = self:ItemLinkToId(itemLink)
							
--~ 							Debug("GetItemCount B", t, s, itemLink, itemID, itemId)
							
							if itemID == itemId then
								
								_, count =  _G.GetGuildBankItemInfo(t, s)
								totalCount = totalCount + count
								
								Debug("GetItemCount C", t, s, itemLink, count, totalCount)
							end
							
							
						end
					end
					

				end
				Debug("GetItemCount D", reqItemLink, totalCount)
				return totalCount
			end
		end
	end]]

	return self.hooks.GetItemCount(...)
end

--[[function core:UseContainerItem(...)
	local container, slot = ...
end]]

--		/run _GBS:SpoofBank()

